/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.app.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.util.Random;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import picto.utils.Picto;
import picto.app.interfaces.Playable;
import picto.utils.ColoredButton;
import picto.utils.Saver;
import picto.app.interfaces.Status;

import static picto.utils.Text._;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class CreatePatternBoard extends JPanel implements Playable{
    private static final long serialVersionUID = 5616895767835624751L;
    private ColoredButton[][] playingBoard;
    private Picto pic;

    // I added this variables, just to avoid all the method calling...
    /**Number of columns*/
    private byte c;
    /**Number of rows*/
    private byte r;

    /** Creates new form CreatePatternBoard */
    public CreatePatternBoard(Picto pic) {
        this.pic = pic;
        pic.regenerate();
        playingBoard = new ColoredButton[pic.getNumberOfRows()]
                                        [pic.getNumberOfColumns()];
        initComponents();
        generateButtonGrid();
        loadButtonGrid();
    }

    /** Creates new form CreatePatternBoard */
    public CreatePatternBoard(final byte i, final byte j) {
        pic = new Picto(i, j);
        playingBoard = new ColoredButton[i][j];
        initComponents();
        generateButtonGrid();
    }


    @Override public Picto getPic() {
        Picto temp = new Picto(r, c);
        temp.setAuthor(pic.getAuthor());
        temp.setTitle(pic.getTitle());
        temp.setDate(pic.getDate());
        for (byte i = 0; i < r; i++){
            for (byte j = 0; j < c; j++){
                temp.setElement(i, j, playingBoard[i][j].getStatus() == Status.FULL);
            }
        }
        return temp;
    }

    private void loadButtonGrid(){
        for (byte i = 0; i < r; i++){
            for (byte j = 0; j < c; j++){
                if (pic.getElement(i, j)){
                    playingBoard[i][j].setStatus(Status.FULL);
                }
            }
        }
    }

    private void generateButtonGrid(){
        c = pic.getNumberOfColumns();
        r = pic.getNumberOfRows();
        setLayout(new GridLayout(r, c));
        PreviewWindow.getInstance().setNewSize(r, c);
        byte i, j;
        for (i = 0; i < r; i++){
            for (j = 0; j < c; j++){
                playingBoard[i][j] = new ColoredButton(i, j);
                add(playingBoard[i][j]);
            }
        }
    }

    public Saver getSaver(){
        Saver temp = new Saver();
        for (byte i = 0; i < r; i++){
            for (byte j = 0; j < c; j++){
                pic.setElement(i, j, playingBoard[i][j].getStatus() == Status.FULL);
            }
        }
        temp.setPic(pic);
        return temp;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 83, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 80, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    @Override public int getXSize() {
        return pic.getNumberOfColumns();
    }

    @Override public int getYSize() {
        return pic.getNumberOfRows();
    }

    @Override public void clean() {
        for (Component but : getComponents()){
            if (but instanceof ColoredButton){
                ((ColoredButton)but).reset();
            }
        }
    }

    @Override public void solve() {
        wtf();
    }

    @Override public void updateHeaderFocus(int i, int j) {

    }

    @Override public void checkBoard(boolean reset_wrong) {
        wtf();
    }

    private void wtf(){
        JOptionPane.showMessageDialog(this
                , _("            "+_("You are creating the board!")+"\n"
                  +_("How do you expect me to know the answer!?"))
                , _("WTF?"), JOptionPane.ERROR_MESSAGE);
        Random rand = new Random();
        for (Component but : getComponents()){
            ((JButton)but).setText("42");
            ((JButton)but).setForeground(new Color(rand.nextInt(256),
                                                   rand.nextInt(256),
                                                   rand.nextInt(256)));
        }
    }

    @Override public void updateColors() {

    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
