/*
 *                      ..::PictoGame::..
 *
 * Copyright (C) Federico Vera 2010-2011 <dktcoding [at] gmail>
 *
 * PictoGame is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or any later
 * version.
 *
 * PictoGame is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package picto.utils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import picto.app.interfaces.Flushable;
import picto.app.interfaces.Locales;
import picto.app.interfaces.Localizable;

/**
 *
 * @author Fido <dktcoding [at] gmail>
 */
public class PhraseScrambler implements Serializable, Localizable, Flushable{
    private static final long serialVersionUID = -5670613584791023L;
    private static PhraseScrambler instance = new PhraseScrambler();
    private static Random rand = new Random(System.nanoTime());
    private static ArrayList<String> list;
    private char[][] obfuscator;
    private String phrase;
    private Picto pic;

    private PhraseScrambler(){
        updateTexts();
        addWeakReference();
    }
    
    public PhraseScrambler (String phrase){
        this.phrase = phrase;
        // The following steps (scramble and obfuscate) can be easily set in
        // just one loop, but I believe is clearer this way, specially
        // considering that we want to obfuscate the phrase and not the code
        scramble();
        obfuscate();
    }

    private void scramble(){
        int length = phrase.split("--")[0].replace(" ", "").length();
        int ySize, xSize;
        
        {// The following is based on solid ... err ... "facts", I demand that I
         // may ... or may not be Vroomfondel
            ySize = 3 + length / 13;
            xSize = 5 + length / 14 + length / ySize;
            int r = rand.nextInt(4);
            xSize -= r;
            ySize += r;
        }
        
        int counter = xSize * ySize - length;
        pic = new Picto((byte)ySize, (byte)xSize);
        int i,j;
        
        do{
            i = rand.nextInt(ySize);
            j = rand.nextInt(xSize);
            if (!pic.getElement(i, j)){
                pic.setElement(i, j, true);
                counter--;
            }
        } while (counter != 0);
        
        pic.regenerate();
    }

    private void obfuscate(){
        obfuscator = new char[pic.getNumberOfRows()][pic.getNumberOfColumns()];
        String foo = phrase.split("--")[0].replace(" ", "").toUpperCase();
        int counter = foo.length();
        
        for (int i = pic.getNumberOfRows() - 1; i >= 0 ; i--){
            for (int j = pic.getNumberOfColumns() - 1; j >= 0 ; j--){
                if (pic.getElement(i, j)){
                    // Get a random upper case char
                    obfuscator[i][j] = (char)(rand.nextInt((int)'Z'-(int)'A')+'A');
                } else {
                    obfuscator[i][j] = foo.charAt(--counter);
                }
            }
        }
    }

    @Override 
    public String toString(){
        int r = pic.getNumberOfRows();
        int c = pic.getNumberOfColumns();
        
        StringBuilder str1 = new StringBuilder(r * (c + 1));
        StringBuilder str2 = new StringBuilder(r * (c + 1));
        
        for (int i = 0; i < r; i++){
            for (int j = 0; j < c; j++){
                str1.append(obfuscator[i][j]);
                str2.append(pic.getElement(i, j) ? '_' : obfuscator[i][j]);
            }
            str1.append('\n');
            str2.append('\n');
        }
        
        String picStr = pic.toString();
        StringBuilder tmp = new StringBuilder(
                picStr.length() + str1.length() + str2.length() + 2);
        
        tmp.append(pic)
           .append('\n')
           .append(str1)
           .append('\n')
           .append(str2);
        
        return tmp.toString();
    }

    public String getPhrase() {
        return phrase;
    }

    public void setPhrase(String phrase) {
        this.phrase = phrase;
    }

    public Picto getPic() {
        return pic;
    }

    public void setPic(Picto pic) {
        this.pic = pic;
    }

    public char getElement(int i, int j){
        return obfuscator[i][j];
    }

    public static PhraseScrambler getRandomScrambler(){
        if (list.isEmpty()){
            instance.updateTexts();
        }
        return new PhraseScrambler(list.remove(rand.nextInt(list.size())));
    }

    public static boolean isEmpty(){
        //@FIXME updateTexts() is called twice... perhaps don't add a reference to Text?
        instance.updateTexts();
        return list.size() <= 0;
    }

    @Override 
    public final void updateTexts() {
        list = new ArrayList<String>(50);
        String lang = Locales.locales[Text.getLocale()].getLanguage();
        BufferedReader reader = new BufferedReader(new InputStreamReader(
                 PhraseScrambler.class.getResourceAsStream("/res/phrases/"+lang)));
        String line;
        
        try {
            line = reader.readLine();
            while (line != null){
                list.add(line);
                line = reader.readLine();
            }
            reader.close();
        } catch (IOException ex) {
            Logger.getLogger(PhraseScrambler.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }

    @Override 
    public final void addWeakReference() {
        Text.addWeakReference(this);
    }

    @Override 
    public void flush() {
        obfuscator = null;
    }
}
